"""Sphinx configuration for aioconsole documentation."""

VERSION = (
    open("../aioconsole/__init__.py").read().split('__version__ = "')[1].split('"')[0]
)

project = "aioconsole"
version = VERSION
author = "Vincent Michel"
copyright = "2020, Vincent Michel"

master_doc = "index"
highlight_language = "python"
extensions = ["sphinx.ext.autodoc"]

html_theme = "sphinx_rtd_theme"
html_context = {
    "display_github": True,
    "github_user": "vxgmichel",
    "github_repo": "aioconsole",
    "github_version": "main",
    "conf_py_path": "/docs/",
    "source_suffix": ".rst",
}

suppress_warnings = ["image.nonlocal_uri"]



"""Run a CLI to simulate dice throws."""

import random
import asyncio
import argparse

from aioconsole import AsynchronousCli


async def dice(reader, writer, faces):
    for _ in range(3):
        await asyncio.sleep(0.33)
        writer.write(".")
        await writer.drain()
    writer.write("\n")
    return random.randint(1, faces)


def main():
    parser = argparse.ArgumentParser(description="Throw a dice.")
    parser.add_argument(
        "--faces", "-f", metavar="N", type=int, default=6, help="Number of faces"
    )
    cli = AsynchronousCli({"dice": (dice, parser)}, prog="dice")

    loop = asyncio.get_event_loop()
    loop.run_until_complete(cli.interact())


if __name__ == "__main__":
    main()



import pytest
import asyncio


def import_uvloop():
    try:
        import uvloop
    except ImportError:
        return None
    else:
        return uvloop


@pytest.fixture(
    params=(
        "default",
        "uvloop",
    ),
)
def event_loop_policy(request):
    if request.param == "default":
        return asyncio.DefaultEventLoopPolicy()
    elif request.param == "uvloop":
        uvloop = import_uvloop()
        if uvloop is None:
            pytest.skip("uvloop is not installed")
        return uvloop.EventLoopPolicy()
    return request.param


@pytest.fixture
def is_uvloop(event_loop_policy):
    uvloop = import_uvloop()
    if uvloop is None:
        return False
    return isinstance(event_loop_policy, uvloop.EventLoopPolicy)



import sys
import asyncio
import collections


async def handle_echo(reader: asyncio.StreamReader, writer: asyncio.StreamWriter):
    loop = asyncio.get_event_loop()
    data = await reader.readline()
    message = data.decode()
    addr = writer.get_extra_info("peername")
    loop.history[addr[0]].append(message.strip())
    writer.write(data)
    await writer.drain()
    writer.close()
    await writer.wait_closed()


async def run(host: str = "localhost", port: int = 8000):
    loop = asyncio.get_event_loop()
    loop.history = collections.defaultdict(list)
    loop.server = await asyncio.start_server(handle_echo, host, port)
    interface = "{}:{}".format(*loop.server.sockets[0].getsockname())
    print(f"The echo service is being served on {interface}")
    async with loop.server:
        await loop.server.serve_forever()


def main(args: list[str]):
    (server,) = args[1:]
    if ":" not in server:
        coro = run(port=int(server))
    else:
        host, port = server.split(":")
        coro = run(host, int(port))
    try:
        asyncio.run(coro)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main(sys.argv)



"""Command line interface for echo server."""

import fnmatch
import asyncio
import argparse

from aioconsole import AsynchronousCli, start_interactive_server
from aioconsole.server import parse_server, print_server

from . import echo


async def get_history(reader, writer, pattern=None):
    history = asyncio.get_event_loop().history
    if not history:
        return "No message in the history"
    if pattern:
        history = {host: history[host] for host in fnmatch.filter(history, pattern)}
    if not history:
        return "No host match the given pattern"
    for host in history:
        writer.write(f"Host {host}:\n".encode())
        for i, message in enumerate(history[host]):
            writer.write(f"  {i}. {message}\n".encode())


def make_cli(streams=None):
    parser = argparse.ArgumentParser(description="Display the message history")
    parser.add_argument("--pattern", "-p", type=str, help="pattern to filter hostnames")
    commands = {"history": (get_history, parser)}
    return AsynchronousCli(commands, streams, prog="echo")


def parse_args(args=None):
    parser = argparse.ArgumentParser(
        description="Run the echo server and a command line interface."
    )
    parser.add_argument(
        "server",
        metavar="[HOST:]PORT",
        type=str,
        help="interface for the echo server, default host is localhost",
    )
    parser.add_argument(
        "--serve-cli",
        metavar="[HOST:]PORT",
        type=str,
        help="serve the command line interface on the given host+port "
        "instead of using the standard streams",
    )
    namespace = parser.parse_args(args)
    host, port = parse_server(namespace.server, parser)
    if namespace.serve_cli is not None:
        serve_cli = parse_server(namespace.serve_cli, parser)
    else:
        serve_cli = None
    return host, port, serve_cli


def main(args=None):
    host, port, serve_cli = parse_args(args)
    if serve_cli:
        cli_host, cli_port = serve_cli
        coro = start_interactive_server(make_cli, cli_host, cli_port)
        server = asyncio.get_event_loop().run_until_complete(coro)
        print_server(server, "command line interface")
    else:
        asyncio.ensure_future(make_cli().interact())
    return echo.run(host, port)


if __name__ == "__main__":
    main()
